@@ -110,7 +110,7 @@ die "Session authentication cannot be used in inetd mode"
 
 # check if the PAM module is available to authenticate
 if (!$config{'no_pam'}) {
-	eval "use Authen::PAM";
+	eval "use Authen::PAM;";
 	if (!$@) {
 		# check if the PAM authentication can be used by opening a
 		# PAM handle
@@ -135,6 +135,12 @@ if (!$config{'no_pam'}) {
 			$pam_msg = "PAM initialization of Authen::PAM failed";
 			}
 		}
+	else {
+		$pam_msg = "Perl module Authen::PAM needed for PAM is not installed : $@";
+		}
+	}
+if ($config{'pam_only'} && !$use_pam) {
+	$pam_msg2 = "PAM use is mandatory, but could not be enabled!";
 	}
 
 # init days and months for http_date
@@ -262,6 +268,13 @@ foreach $a (split(/\s+/, $config{'anonymous'})) {
 		}
 	}
 
+# build IP access list
+foreach $a (split(/\s+/, $config{'ipaccess'})) {
+	if ($a =~ /^([^=]+)=(\S+)$/) {
+		$ipaccess{$1} = $2;
+		}
+	}
+
 # build unauthenticated URLs list
 @unauth = split(/\s+/, $config{'unauth'});
 
@@ -293,6 +306,9 @@ if ($config{'extauth'}) {
 # Re-direct STDERR to a log file
 if ($config{'errorlog'} ne '-') {
 	open(STDERR, ">>$config{'errorlog'}") || die "failed to open $config{'errorlog'} : $!";
+	if ($config{'logperms'}) {
+		chmod(oct($config{'logperms'}), $config{'errorlog'});
+		}
 	}
 
 # Init allow and deny lists
@@ -413,6 +429,7 @@ open(STDIN, "</dev/null");
 open(STDOUT, ">/dev/null");
 &log_error("miniserv.pl started");
 &log_error($pam_msg) if ($pam_msg);
+&log_error($pam_msg2) if ($pam_msg2);
 
 # write out the PID file
 open(PIDFILE, "> $config{'pidfile'}");
@@ -788,6 +805,9 @@ if (!$reqline && (!$use_ssl || $checked_timeout > 1)) {
 	# An empty request .. just close the connection
 	return 0;
 	}
+elsif ($reqline =~ /^(SEARCH|PUT)\s+/) {
+	&http_error(400, "Bad Request");
+	}
 elsif ($reqline !~ /^(GET|POST|HEAD)\s+(.*)\s+HTTP\/1\..$/) {
 	if ($use_ssl) {
 		# This could be an http request when it should be https
@@ -1030,6 +1050,19 @@ if (%users) {
 			}
 		}
 
+	if (!$validated && !$deny_authentication) {
+		# check for IP-based authentication
+		local $a;
+		foreach $a (keys %ipaccess) {
+			if ($acptip eq $a) {
+				# It does! Auth as the user
+				$validated = 3;
+				$baseauthuser = $authuser =
+					$ipaccess{$a};
+				}
+			}
+		}
+
 	# Check for normal HTTP authentication
 	if (!$validated && !$deny_authentication && !$config{'session'} &&
 	    $header{authorization} =~ /^basic\s+(\S+)$/i) {
@@ -1181,12 +1214,16 @@ if (%users) {
 				# to force display of password change form.
 				$validated = 1;
 				$authuser = undef;
-				$querystring = "&user=".&urlize($vu);
+				$querystring = "&user=".&urlize($vu).
+					       "&pam=".$use_pam;
 				$method = "GET";
 				$queryargs = "";
 				$page = $config{'password_form'};
 				$logged_code = 401;
 				$miniserv_internal = 2;
+				syslog("crit",
+					"Expired login as $in{'user'} ".
+					"from $acpthost") if ($use_syslog);
 				}
 			else {
 				# Login failed, or password has expired. Display
@@ -1267,9 +1304,13 @@ if (%users) {
 		local $a;
 		foreach $a (keys %anonymous) {
 			if (substr($simple, 0, length($a)) eq $a) {
-				# It does! Auth as the user ..
-				$validated = 3;
-				$baseauthuser = $authuser = $anonymous{$a};
+				# It does! Auth as the user, if IP access
+				# control allows him.
+				if (&check_user_ip($anonymous{$a})) {
+					$validated = 3;
+					$baseauthuser = $authuser =
+						$anonymous{$a};
+					}
 				}
 			}
 		}
@@ -1362,8 +1403,7 @@ if (%users) {
 		}
 
 	# Check per-user IP access control
-	if ($deny{$baseauthuser} && &ip_match($acptip, $localip, @{$deny{$baseauthuser}}) ||
-	    $allow{$baseauthuser} && !&ip_match($acptip, $localip, @{$allow{$baseauthuser}})) {
+	if (!&check_user_ip($baseauthuser)) {
 		&http_error(403, "Access denied for $acptip");
 		return 0;
 		}
@@ -1648,7 +1688,13 @@ if (&get_type($full) eq "internal/cgi") {
 
 		if ($config{'log'}) {
 			open(MINISERVLOG, ">>$config{'logfile'}");
-			chmod(0600, $config{'logfile'});
+			if ($config{'logperms'}) {
+				chmod(oct($config{'logperms'}),
+				      $config{'logfile'});
+				}
+			else {
+				chmod(0600, $config{'logfile'});
+				}
 			}
 		$doing_eval = 1;
 		$main_process_id = $$;
@@ -2484,9 +2530,24 @@ if ($use_pam) {
 				      \&pam_conv_func);
 	if (ref($pamh)) {
 		local $pam_ret = $pamh->pam_authenticate();
-		return 1 if ($pam_ret == PAM_SUCCESS);
+		if ($pam_ret == PAM_SUCCESS()) {
+			# Logged in OK .. make sure password hasn't expired
+			local $acct_ret = $pamh->pam_acct_mgmt();
+			if ($acct_ret == PAM_SUCCESS()) {
+				return 1;
+				}
+			elsif ($acct_ret == PAM_NEW_AUTHTOK_REQD() ||
+			       $acct_ret == PAM_ACCT_EXPIRED()) {
+				return 2;
+				}
+			}
+		return 0;
 		}
 	}
+elsif ($config{'pam_only'}) {
+	# Pam is not available, but configuration forces it's use!
+	return 0;
+	}
 elsif ($config{'passwd_file'}) {
 	local $rv = 0;
 	open(FILE, $config{'passwd_file'});
@@ -2522,6 +2583,10 @@ elsif ($config{'passwd_file'}) {
 						$rv = 2;
 						}
 					}
+				if ($p eq "" && $config{'passwd_blank'}) {
+					# Force password change
+					$rv = 2;
+					}
 				last;
 				}
 			}
@@ -2625,3 +2690,14 @@ foreach $s (@socketfhs) {
 	}
 }
 
+sub check_user_ip
+{
+if ($deny{$_[0]} &&
+    &ip_match($acptip, $localip, @{$deny{$_[0]}}) ||
+    $allow{$_[0]} &&
+    !&ip_match($acptip, $localip, @{$allow{$_[0]}})) {
+	return 0;
+	}
+return 1;
+}
+
