@@ -263,7 +263,7 @@ if ($remote_user && @_ > 1) {
 			}
 		}
 	if (!$ENV{'LOCAL_USER'}) {
-		print qq~<td width="84" nowrap><a href='$logout'><img src="$gconfig{'webprefix'}/images/top_bar/$loicon" width=$lowidth height="31" border="0" alt="$lotext"></td>~;
+		print qq~<td width="84" nowrap><a href='$gconfig{'webprefix'}$logout'><img src="$gconfig{'webprefix'}/images/top_bar/$loicon" width=$lowidth height="31" border="0" alt="$lotext"></td>~;
 		}
 	print qq~<td width="3" nowrap>
 	      <div align="right"><img src="$gconfig{'webprefix'}/images/top_bar/right.jpg" width="3" height="32"></div>
@@ -385,7 +385,7 @@ if (@_ > 1 && !$one && $remote_user) {
                 print qq~</a></center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~; 
               } else {
-               print qq~<td nowrap><a href=$gconfig{'webprefix'}/?cat=$c><center><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t"><br>~;
 
             &chop_font;
 
@@ -427,8 +427,9 @@ if (@_ > 1 && (!$_[5] || $ENV{'HTTP_WEBMIN_SERVERS'})) {
 	}
 	if (!$_[4]) {
 		local $mi = $module_index_link || "/$module_name/";
+		local $mt = $module_index_name || $text{'header_module'};
 		&tab_start; print "<a href=\"$gconfig{'webprefix'}$mi\">",
-			    "$text{'header_module'}</a>"; &tab_end;
+			    "$mt</a>"; &tab_end;
 		}
 	if (ref($_[2]) eq "ARRAY") {
 		&tab_start; print &hlink($text{'header_help'}, $_[2]->[0], $_[2]->[1]); &tab_end;
@@ -469,11 +470,6 @@ print qq~
     if (!$_[5]) {
 	    # Show page title in tab
 	    local $title = $_[0];
-	    $title =~ s/&auml;//g;
-	    $title =~ s/&ouml;//g;
-	    $title =~ s/&uuml;//g;
-	    $title =~ s/&nbsp;/ /g;
-
 	    print "<p><table border=0 cellpadding=0 cellspacing=0 width=95% align=center><tr><td><table border=0 cellpadding=0 cellspacing=0 height=20><tr>\n";
 	    $usercol = defined($gconfig{'cs_header'}) ||
 		   defined($gconfig{'cs_table'}) ||
@@ -528,7 +524,7 @@ if ($theme_no_table) {
 	print "</ul>\n";
 	}
 elsif (@header_arguments > 1 && !$header_arguments[5]) {
-	print "</table></table><br>\n";
+	print "&nbsp;\n</table></table><br>\n";
 	}
 
 print "<table border=0 width=100% align=center cellspacing=0 cellpadding=0><tr><td>\n";
@@ -599,7 +595,7 @@ sub chop_font {
 if (!$current_lang_info->{'titles'} || $gconfig{'texttitles'}) {
 	print $t;
 } else {
-        foreach $l (split(//, $t)) {
+        foreach $l (split(//, &entities_to_ascii($t))) {
             $ll = ord($l);
 	    local $gif;
             if ($ll > 127 && $current_lang_info->{'charset'}) {
@@ -638,5 +634,45 @@ sub tab_end {
     </td>~;
 }
 
+sub theme_ui_post_header
+{
+local ($text) = @_;
+local $rv;
+if (defined($text)) {
+	$rv .= "<center><font size=+1>$text</font></center>\n";
+	}
+if ($main::printed_unbuffered_header) {
+	$rv .= "<hr>\n";
+	}
+else {
+	$rv .= "<br>\n";
+	}
+return $rv;
+}
+
+sub theme_ui_pre_footer
+{
+local $rv;
+if ($main::printed_unbuffered_header) {
+	$rv .= "<hr>\n";
+	}
+return $rv;
+}
+
+sub theme_error
+{
+&ui_print_header(undef, $text{'error'}, "");
+print "<h3>",($main::whatfailed ? "$main::whatfailed : " : ""),@_,"</h3>\n";
+&ui_print_footer();
+}
+
+sub theme_ui_print_unbuffered_header
+{
+$| = 1;
+$theme_no_table = 1;
+$main::printed_unbuffered_header = 1;
+&ui_print_header(@_);
+}
+
 1;
 
